@@ -200,6 +200,38 @@ class QueryTests(JSONFieldTestCase):
 
         assert "Lookup 'range' doesn't work with JSONField" in str(excinfo.value)
 
+    def test_in_lookup(self):
+        objs = list(
+            JSONModel.objects.filter(attrs__in=[{"a": "c"}, {"a": "b"}, {"c": "b"}])
+        )
+        assert objs == [self.objs[0]]
+
+        objs = list(JSONModel.objects.filter(attrs__a__in=["b"]))
+        assert objs == [self.objs[0]]
+
+        objs = list(JSONModel.objects.filter(attrs__in=[111, 1337, 333]))
+        assert objs == [self.objs[1]]
+
+        objs = list(
+            JSONModel.objects.filter(
+                attrs__in=[["array"], ["an", "array"], ["another", "array"]]
+            )
+        )
+        assert objs == [self.objs[2]]
+
+        objs = list(JSONModel.objects.filter(attrs__in=["bar", "foo", "baz"]))
+        assert objs == [self.objs[4]]
+
+        objs = list(
+            JSONModel.objects.filter(
+                attrs__in=[{"a": "b"}, 1337, "foo", ["an", "array"]]
+            )
+        )
+        assert objs == [self.objs[0], self.objs[1], self.objs[2], self.objs[4]]
+
+        objs = list(JSONModel.objects.filter(attrs__in=["bar"]))
+        assert objs == []
+
 
 class ArrayQueryTests(JSONFieldTestCase):
     def setUp(self):
@@ -252,27 +284,30 @@ class ExtraLookupsQueryTests(JSONFieldTestCase):
         self.objs = [
             JSONModel.objects.create(attrs={}),
             JSONModel.objects.create(
+                name='"b"',
                 attrs={
                     "a": "b",
                     "c": 1,
                     9001: 9002,
                     '"': "awkward",
                     "\n": "super awkward",
-                }
+                },
             ),
             JSONModel.objects.create(
+                name="b",
                 attrs={
                     "a": "b",
                     "c": 1,
-                    "d": ["e", {"f": "g"}],
+                    "d": ["e", {"f": "g"}, 1],
                     "h": True,
                     "i": False,
                     "j": None,
                     "k": {"l": "m"},
-                }
+                },
             ),
             JSONModel.objects.create(attrs=[1, [2]]),
             JSONModel.objects.create(attrs={"k": True, "l": False, "\\": "awkward"}),
+            JSONModel.objects.create(attrs={"a": "n"}, name="x"),
         ]
 
     def test_has_key_invalid_type(self):
@@ -287,6 +322,7 @@ class ExtraLookupsQueryTests(JSONFieldTestCase):
         assert list(JSONModel.objects.filter(attrs__has_key="a")) == [
             self.objs[1],
             self.objs[2],
+            self.objs[5],
         ]
 
     def test_has_key_2(self):
@@ -331,6 +367,7 @@ class ExtraLookupsQueryTests(JSONFieldTestCase):
             self.objs[1],
             self.objs[2],
             self.objs[4],
+            self.objs[5],
         ]
 
     def test_has_any_keys_awkward(self):
@@ -338,19 +375,79 @@ class ExtraLookupsQueryTests(JSONFieldTestCase):
             self.objs[4]
         ]
 
-    def test_contains(self):
+    def test_contains_scalar_in_scalar(self):
+        assert list(JSONModel.objects.filter(attrs__a__contains="b")) == [
+            self.objs[1],
+            self.objs[2],
+        ]
+        assert list(JSONModel.objects.filter(attrs__c__contains=1)) == [
+            self.objs[1],
+            self.objs[2],
+        ]
+        assert list(JSONModel.objects.filter(attrs__l__contains=False)) == [
+            self.objs[4],
+        ]
+        assert list(JSONModel.objects.filter(attrs__h__contains=True)) == [
+            self.objs[2],
+        ]
+        assert list(JSONModel.objects.filter(attrs__j__contains=None)) == [
+            self.objs[2],
+        ]
+
+        assert list(JSONModel.objects.filter(attrs__a__contains=1)) == []
+        assert list(JSONModel.objects.filter(attrs__c__contains="b")) == []
+        assert list(JSONModel.objects.filter(attrs__l__contains=True)) == []
+        assert list(JSONModel.objects.filter(attrs__h__contains=False)) == []
+
+    def test_contains_object_in_object(self):
         assert list(JSONModel.objects.filter(attrs__contains={"a": "b"})) == [
             self.objs[1],
             self.objs[2],
         ]
-
-    def test_contains_2(self):
-        assert list(JSONModel.objects.filter(attrs__contains={"l": False})) == [
-            self.objs[4]
+        assert list(JSONModel.objects.filter(attrs__contains={"c": 1})) == [
+            self.objs[1],
+            self.objs[2],
         ]
+        assert list(JSONModel.objects.filter(attrs__contains={"l": False})) == [
+            self.objs[4],
+        ]
+        assert list(JSONModel.objects.filter(attrs__contains={"h": True})) == [
+            self.objs[2],
+        ]
+        assert list(JSONModel.objects.filter(attrs__contains={"j": None})) == [
+            self.objs[2],
+        ]
+
+        assert list(JSONModel.objects.filter(attrs__contains={"a": 1})) == []
+        assert list(JSONModel.objects.filter(attrs__contains={"c": "b"})) == []
+        assert list(JSONModel.objects.filter(attrs__contains={"l": True})) == []
+        assert list(JSONModel.objects.filter(attrs__contains={"h": False})) == []
 
     def test_contains_array(self):
         assert list(JSONModel.objects.filter(attrs__contains=[[2]])) == [self.objs[3]]
+        assert list(JSONModel.objects.filter(attrs__contains=1)) == [self.objs[3]]
+        assert list(JSONModel.objects.filter(attrs__d__contains="e")) == [self.objs[2]]
+        assert list(JSONModel.objects.filter(attrs__d__contains=1)) == [self.objs[2]]
+        assert list(JSONModel.objects.filter(attrs__d__contains={"f": "g"})) == [
+            self.objs[2]
+        ]
+        assert list(JSONModel.objects.filter(attrs__d__contains=2)) == []
+
+    def test_contains_expression_and_subqueries(self):
+        with pytest.raises(ValueError) as excinfo:
+            list(JSONModel.objects.filter(attrs__a__contains=F("name")))
+        assert (
+            str(excinfo.value)
+            == "JSONField's 'contains' lookup only works with literal values"
+        )
+
+        subquery = JSONModel.objects.filter(name="b").values_list("attrs")
+        with pytest.raises(ValueError) as excinfo:
+            JSONModel.objects.filter(attrs__contains=subquery)
+        assert (
+            str(excinfo.value)
+            == "JSONField's 'contains' lookup only works with literal values"
+        )
 
     def test_contained_by(self):
         assert list(
@@ -418,6 +515,41 @@ class ExtraLookupsQueryTests(JSONFieldTestCase):
             JSONModel.objects.filter(id__in=JSONModel.objects.filter(attrs__c=1))
         ) == [self.objs[1], self.objs[2]]
 
+    def test_in_lookup(self):
+        objs = list(JSONModel.objects.filter(attrs__a__in=["x", "b", "y"]))
+        assert objs == [self.objs[1], self.objs[2]]
+
+        objs = list(
+            JSONModel.objects.filter(
+                attrs__d__1__in=[{"x": "g"}, {"f": "g"}, {"f": "x"}]
+            )
+        )
+        assert objs == [self.objs[2]]
+
+        objs = list(JSONModel.objects.filter(attrs__k__in=[True, {"l": "m"}, "x"]))
+        assert objs == [self.objs[2], self.objs[4]]
+
+        objs = list(JSONModel.objects.filter(attrs__k__in=[{"l": "m"}, "x"]))
+        assert objs == [self.objs[2]]
+
+        objs = list(JSONModel.objects.filter(attrs__k__in=[True, "x"]))
+        assert objs == [self.objs[4]]
+
+        with pytest.raises(ValueError) as excinfo:
+            JSONModel.objects.filter(attrs__a__in=[F("name"), "n"])
+        assert (
+            str(excinfo.value)
+            == "JSONField's 'in' lookup only works with literal values"
+        )
+
+        with pytest.raises(ValueError) as excinfo:
+            subquery = JSONModel.objects.filter(name="b").values_list("attrs")
+            JSONModel.objects.filter(attrs__in=subquery)
+        assert (
+            str(excinfo.value)
+            == "JSONField's 'in' lookup only works with literal values"
+        )
+
 
 class TestCheck(JSONFieldTestCase):
 
